@@ -1,5 +1,5 @@
 /*
-    $Id: kcalcdisplay.cpp,v 1.16.2.2 2004/09/25 08:58:25 kniederk Exp $
+    $Id: kcalcdisplay.cpp,v 1.30 2005/01/22 13:42:57 wheeler Exp $
 
     KCalc, a scientific calculator for the X window system using the
     Qt widget libraries, available at no cost at http://www.troll.no
@@ -27,7 +27,9 @@
 #include <ctype.h>
 
 #include <qclipboard.h>
+#include <qregexp.h>
 
+#include <kglobal.h>
 #include <klocale.h>
 #include <knotifyclient.h>
 #include "kcalcdisplay.h"
@@ -51,10 +53,10 @@ static CALCAMNT toDouble(const QString &s, bool &ok)
 }
 
 KCalcDisplay::KCalcDisplay(QWidget *parent, const char *name)
-  :QLabel(parent,name), _button(0), _lit(false),
+  :QLabel(parent,name), _beep(false), _groupdigits(false), _button(0), _lit(false),
    _num_base(NB_DECIMAL), _input_limit(0),
    _display_size(DEC_SIZE), _precision(9),
-   _fixed_precision(-1), selection_timer(new QTimer)
+   _fixed_precision(-1), _error(false), selection_timer(new QTimer)
 {
 	setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
 	setAlignment(AlignRight | AlignVCenter);
@@ -67,7 +69,7 @@ KCalcDisplay::KCalcDisplay(QWidget *parent, const char *name)
 	connect(selection_timer, SIGNAL(timeout()),
 		this, SLOT(slotSelectionTimedOut()));
 
-	Reset();
+	sendEvent(EventReset);
 }
 
 KCalcDisplay::~KCalcDisplay()
@@ -75,6 +77,39 @@ KCalcDisplay::~KCalcDisplay()
 	delete selection_timer;
 }
 
+bool KCalcDisplay::sendEvent(Event const event)
+{
+	switch(event)
+	{
+	case EventReset:
+		_error = false;
+		_display_amount = 0.0;
+		_str_int = "0";
+		_str_int_exp = (char *)0;
+
+		_eestate = false;
+		_period = false;
+		_neg_sign = false;
+
+		updateDisplay();
+
+		return true;
+	case EventClear:
+		if (! _error) return sendEvent(EventReset);
+		return false;
+	case EventChangeSign:
+		return changeSign();
+	case EventError:
+		_error = true;
+		updateDisplay();
+
+		return true;
+	default:
+		return false;
+	}
+}
+
+
 void KCalcDisplay::slotCut(void)
 {
 	if(_error  &&  _beep)
@@ -84,7 +119,7 @@ void KCalcDisplay::slotCut(void)
 	}
 
 	slotCopy();
-	Reset();
+	sendEvent(EventReset);
 }
 
 void KCalcDisplay::slotCopy(void)
@@ -112,25 +147,29 @@ void KCalcDisplay::slotPaste(bool bClipboard)
 		return;
 	}
 
-	bool was_ok;
-	CALCAMNT tmp_result;
-	
 	if (_num_base == NB_HEX  &&  ! tmp_str.startsWith("0x", false))
 	  tmp_str.prepend( "0x" );
-	
+
+	bool was_ok;
+	CALCAMNT tmp_result;
+
 	if ( (_num_base == NB_OCTAL || _num_base == NB_BINARY) &&
 	     !  tmp_str.startsWith("0x",false))
-	  tmp_result = CALCAMNT(tmp_str.toLongLong(& was_ok, _num_base));
-	else
-	  tmp_result = toDouble(tmp_str, was_ok);
-	
-	if (!was_ok)
 	{
-		tmp_result = (CALCAMNT) (0);
-		if(_beep) KNotifyClient::beep();		
+		tmp_result = CALCAMNT(tmp_str.toLongLong(& was_ok, _num_base));
+		if (!was_ok)
+		{
+			tmp_result = (CALCAMNT) (0);
+			if(_beep) KNotifyClient::beep();
+			return ;
+		}
+	  
+		setAmount(tmp_result);
+	} 
+	else
+	{
+		setAmount(tmp_str);
 	}
-
-	setAmount(tmp_result);
 }
 
 void KCalcDisplay::slotDisplaySelected(void)
@@ -195,15 +234,38 @@ void KCalcDisplay::setBeep(bool flag)
 	_beep = flag;
 }
 
+void KCalcDisplay::setGroupDigits(bool flag)
+{
+	_groupdigits = flag;
+}
+
 CALCAMNT KCalcDisplay::getAmount(void) const
 {
 	return _display_amount;
 }
 
+bool KCalcDisplay::setAmount(const QString &string)
+{
+	bool was_ok;
+	CALCAMNT tmp_result;
+
+	tmp_result = toDouble(string, was_ok);
+	
+	if (!was_ok)
+	{
+		tmp_result = (CALCAMNT) (0);
+		if(_beep) KNotifyClient::beep();
+		return false;
+	}
+	
+	setAmount(tmp_result);
+
+	return true;
+}
+
 bool KCalcDisplay::setAmount(CALCAMNT new_amount)
 {
-	if(_error)
-		return false;
+	if(_error) return false;
 
 	QString display_str;
 
@@ -223,7 +285,10 @@ bool KCalcDisplay::setAmount(CALCAMNT new_amount)
 		MODF(_display_amount, &tmp_round);
 
 		if (tmp_round < KCALC_LONG_MIN || tmp_round > KCALC_ULONG_MAX)
-			_error = true;
+		{
+			sendEvent(EventError);
+			return false;
+		}
 
 		//
 		// We may be in that never-never land where boh numbers
@@ -243,7 +308,11 @@ bool KCalcDisplay::setAmount(CALCAMNT new_amount)
 		}
 
 		display_str = QString::number(rounded_num, _num_base).upper();
-		if (display_str.length() > DSP_SIZE) _error = true;
+		if (display_str.length() > DSP_SIZE)
+		{
+			sendEvent(EventError);
+			return false;
+		}
 	}
 	else // _num_base == NB_DECIMAL
 	{
@@ -257,27 +326,62 @@ bool KCalcDisplay::setAmount(CALCAMNT new_amount)
 		else
 			display_str = QCString().sprintf(PRINT_LONG_BIG, _precision, _display_amount);
 
-		if (display_str.length() > DSP_SIZE) _error = true;
+		if (display_str.length() > DSP_SIZE)
+		{
+			sendEvent(EventError);
+			return false;
+		}
 	}
 
-	if(_error)
-	{
-		if(_beep) KNotifyClient::beep();
-		setText(i18n("Error"));
-		return false;
-	}
-	else
-	{
-		setText(display_str);
-		return true;
-	}
+	Q_ASSERT(_error == false);
+	setText(display_str);
+	return true;
 	
 }
 
 void KCalcDisplay::setText(QString const &string)
 {
-	QLabel::setText(string);
-	emit changedText(string);
+	QString localizedString = string;
+
+	// Only treat digits (skip the sign)
+	int first_digit = localizedString.find(QRegExp("\\d"));
+	
+	// If we aren't in decimal mode, we don't need to modify the string
+	if (_num_base == NB_DECIMAL  &&  !_error
+	    &&  _groupdigits  &&  first_digit != -1)
+	{
+		// Obtain decimal symbol and thousands separator
+		QString decimalSymbol = KGlobal::locale()->decimalSymbol();
+		QString thousandsSeparator = KGlobal::locale()->thousandsSeparator();
+		// Replace dot with locale decimal separator
+		localizedString.replace(QChar('.'),decimalSymbol);
+	
+		// Insert the thousand separators
+                int decimalSymbolPos = localizedString.find(decimalSymbol);
+                int eSymbolPos = localizedString.find('e');
+
+                if (-1 == decimalSymbolPos)
+                    decimalSymbolPos = localizedString.length();
+                if (-1 == eSymbolPos)
+                    eSymbolPos = localizedString.length();
+
+                int pos = QMIN((decimalSymbolPos - 1), (eSymbolPos-1));
+                
+		for(int index=1;pos>first_digit;pos--,index++)
+			if (index%3 == 0)
+				localizedString.insert(pos,thousandsSeparator);
+	}
+
+	QLabel::setText(localizedString);
+	emit changedText(localizedString);
+}
+
+QString KCalcDisplay::text() const
+{
+	if (_num_base != NB_DECIMAL || _error)
+		return QLabel::text();
+
+	return QCString().sprintf(PRINT_LONG_BIG, 40, _display_amount);
 }
 
 /* change representation of display to new base (i.e. binary, decimal,
@@ -590,46 +694,7 @@ bool KCalcDisplay::changeSign(void)
 	return true;
 }
 
-bool KCalcDisplay::clearLastInput(void)
-{
-	if (_error)
-	{
-		return updateDisplay();
-	}
-	else
-	{
-		_period	= false;
-		_eestate = false;
-		_neg_sign = false;
-		_str_int = "0";
-		_str_int_exp = (char *)0;
-		return updateDisplay();
-	}
-}
-
-void KCalcDisplay::setError(bool error)
-{
-	_error = error;
-	updateDisplay();
-}
-
 bool KCalcDisplay::getError(void) const
 {
 	return _error;
 }
-
-void KCalcDisplay::Reset(void)
-{
-	// This updates display through setError.
-	// Should one explicitly call UpdateDisplay to be nicer?
-
-	_display_amount = 0.0;
-	_str_int = "0";
-	_str_int_exp = (char *)0;
-
-	_eestate = false;
-	_period = false;
-	_neg_sign = false;
-
-	setError(false);
-}
