@@ -1,5 +1,5 @@
 /*
-    $Id: kcalctype.h,v 1.18.2.1 2004/03/12 12:04:39 waba Exp $
+    $Id: kcalctype.h,v 1.25 2005/02/24 09:21:01 faure Exp $
 
     KCalc, a scientific calculator for the X window system using the
     Qt widget libraries, available at no cost at http://www.troll.no
@@ -31,19 +31,10 @@
 #define KCALC_TYPE_H
 
 #include <limits.h>
+#include <float.h>
 #include <math.h>
 
-// The following for all the poor devels out there who don't have long double math.
-// I guess it's time to switch to LINUX guys .....
-
-// Undefine HAVE_LONG_DOUBLE for Beta 4 since RedHat 5.0 comes with a borken
-// glibc
-
-//#ifdef HAVE_LONG_DOUBLE
-//#undef HAVE_LONG_DOUBLE
-//#endif
-
-#ifdef HAVE_LONG_DOUBLE
+#if defined(HAVE_LONG_DOUBLE) && defined(HAVE_L_FUNCS)
 /* should be detected by autoconf and defined in config.h
    Be carefull when modifying these lines. HAVE_LONG_DOUBLE
    is used all over kcalc's sources to determine whether 
@@ -58,7 +49,7 @@
 
 
 
-#ifdef HAVE_LONG_DOUBLE
+#if defined(HAVE_LONG_DOUBLE) && defined(HAVE_L_FUNCS)
 	#define FABS(X)		fabsl(X)
 	#define MODF(X,Y)	modfl(X,Y)
 	#define FMOD(X,Y)	fmodl(X,Y)
@@ -79,8 +70,11 @@
 	#define LN(X)		logl(X)
 	#define LOG_TEN(X)	log10l(X)
 	#define SQRT(X)		sqrtl(X)
+	#define CBRT(X)		cbrtl(X)
 	#define ISINF(X)	isinfl(X)
 	#define STRTOD(X,Y)	strtold(X,Y)
+	#define ROUND(X)	roundl(X)
+	#define CALCAMNT_EPSILON	LDBL_EPSILON
 #else
 	#define FABS(X)		fabs(X)
 	#define MODF(X,Y)	modf(X,Y)
@@ -102,8 +96,16 @@
 	#define LN(X)		log(X)
 	#define LOG_TEN(X)	log10(X)
 	#define SQRT(X)		sqrt(X)
+	#define CBRT(X)		cbrt(X)
 	#define ISINF(X)	isinf(X)
 	#define STRTOD(X,Y)	strtod(X,Y)
+	#define ROUND(X)	round(X)
+	#define CALCAMNT_EPSILON	DBL_EPSILON
+#endif
+
+#if !defined(HAVE_FUNC_ROUND) && !defined(HAVE_FUNC_ROUNDL)
+	#undef	ROUND
+	#define ROUND(X)	rint(X)
 #endif
 
 #undef HAVE_LONG_LONG
