@@ -1,8 +1,8 @@
 /*
     KFloppy
-    Copyright (C) 1997 Bernd Johannes Wuebben
-                       wuebben@math.cornell.edu
-
+    
+    Copyright (C) 1997 Bernd Johannes Wuebben <wuebben@math.cornell.edu>
+    Copyright (C) 2004 Nicolas GOUTTE <goutte@kde.org>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,12 +23,10 @@
 
 #include <qlayout.h>
 #include <qcheckbox.h>
-#include <qlineedit.h>
 #include <qlabel.h>
 #include <qcursor.h>
 #include <qradiobutton.h>
 #include <qbuttongroup.h>
-#include <qcombobox.h>
 
 #include <kconfig.h>
 
@@ -40,13 +38,15 @@
 #include <kapplication.h>
 #include <kprogress.h>
 #include <klocale.h>
+#include <kcombobox.h>
+#include <klineedit.h>
 
-#include "floppy.moc"
+#include "floppy.h"
 #include "format.h"
 
 FloppyData::FloppyData(QWidget * parent, const char * name)
  : KDialog( parent, name ),
-	formatActions(0L)
+	formatActions(0L), m_canLowLevel(false)
 {
 
 	formating = false;
@@ -70,8 +70,8 @@ FloppyData::FloppyData(QWidget * parent, const char * name)
         g1->addWidget( label1, 0, 0, AlignLeft );
 
 
-	deviceComboBox = new QComboBox( false, this, "ComboBox_1" );
-	g1->addWidget( deviceComboBox, 0, 1, AlignLeft );
+	deviceComboBox = new KComboBox( false, this, "ComboBox_1" );
+	g1->addWidget( deviceComboBox, 0, 1 );
 
 	deviceComboBox->insertItem(i18n("Primary"));
 	deviceComboBox->insertItem(i18n("Secondary"));
@@ -80,9 +80,10 @@ FloppyData::FloppyData(QWidget * parent, const char * name)
 	label2->setText(i18n("Size:"));
         g1->addWidget( label2, 1, 0, AlignLeft );
 
-	densityComboBox = new QComboBox( false, this, "ComboBox_1" );
-	g1->addWidget( densityComboBox, 1, 1, AlignLeft );
+	densityComboBox = new KComboBox( false, this, "ComboBox_1" );
+	g1->addWidget( densityComboBox, 1, 1 );
 
+	densityComboBox->insertItem( i18n( "Auto-Detect" ) );
 	densityComboBox->insertItem(i18n("3.5\" 1.44MB"));
 	densityComboBox->insertItem(i18n("3.5\" 720KB"));
 	densityComboBox->insertItem(i18n("5.25\" 1.2MB"));
@@ -93,15 +94,60 @@ FloppyData::FloppyData(QWidget * parent, const char * name)
 	label3->setText(i18n("File system:"));
         g1->addWidget( label3, 2, 0, AlignLeft );
 
-	filesystemComboBox = new QComboBox( false, this, "ComboBox_2" );
-	g1->addWidget( filesystemComboBox, 2, 1, AlignLeft );
+	filesystemComboBox = new KComboBox( false, this, "ComboBox_2" );
+	g1->addWidget( filesystemComboBox, 2, 1 );
+	g1->setColStretch(1, 1);
 
-	filesystemComboBox->insertItem(i18n("DOS"));
-#ifdef ANY_LINUX
-	filesystemComboBox->insertItem(i18n("ext2"));
-#endif
-#ifdef ANY_BSD
-	filesystemComboBox->insertItem(i18n("UFS"));
+        // If you modify the user visible string, change them also (far) below
+
+        QString userFeedBack;
+        uint numFileSystems = 0;
+
+#if defined(ANY_LINUX)
+        if (FATFilesystem::runtimeCheck()) {
+            filesystemComboBox->insertItem(i18n("DOS"));
+            ++numFileSystems;
+            userFeedBack += i18n( "Linux", "Program mkdosfs found." );
+        }
+        else {
+            userFeedBack += i18n( "Linux", "Program mkdosfs <b>not found</b>. MSDOS formatting <b>not available</b>." );
+        }
+        userFeedBack += "<br>";
+        if (Ext2Filesystem::runtimeCheck()) {
+            filesystemComboBox->insertItem(i18n("ext2"));
+            ++numFileSystems;
+            userFeedBack += i18n( "Linux", "Program mke2fs found." );
+        }
+        else {
+            userFeedBack += i18n( "Linux", "Program mke2fs <b>not found</b>. Ext2 formatting <b>not available</b>" );
+        }
+        userFeedBack += "<br>";
+        if (MinixFilesystem::runtimeCheck()) {
+            filesystemComboBox->insertItem(i18n("Minix"));
+            ++numFileSystems;
+            userFeedBack += i18n( "Linux", "Program mkfs.minix found." );
+        }
+        else {
+            userFeedBack += i18n( "Linux", "Program mkfs.minix <b>not found</b>. Minix formatting <b>not available</b>" );
+        }
+#elif defined(ANY_BSD)
+        if (FATFilesystem::runtimeCheck()) {
+            filesystemComboBox->insertItem(i18n("DOS"));
+            ++numFileSystems;
+            userFeedBack += i18n( "BSD", "Program newfs_msdos found." );
+        }
+        else {
+            userFeedBack += i18n( "BSD", "Program newfs_msdos <b>not found</b>. MSDOS formatting <b>not available</b>." );
+        }
+        userFeedBack += "<br>";
+        if (UFSFilesystem::runtimeCheck()) {
+            filesystemComboBox->insertItem(i18n("UFS"));
+            ++numFileSystems;
+            userFeedBack += i18n( "BSD", "Program newfs found." );
+        }
+        else {
+            userFeedBack += i18n( "BSD", "Program newfs <b>not found</b>. UFS formatting <b>not available</b>." );
+        }
 #endif
 
         v1->addSpacing( 10 );
@@ -118,9 +164,21 @@ FloppyData::FloppyData(QWidget * parent, const char * name)
 
 	fullformat = new QRadioButton( buttongroup, "RadioButton_3" );
 	fullformat->setText(i18n( "Fu&ll format") );
-	fullformat->setChecked(true);
         v2->addWidget( fullformat, AlignLeft );
 
+        // ### TODO: we need some user feedback telling why full formatting is disabled.
+        userFeedBack += "<br>";
+        m_canLowLevel = FDFormat::runtimeCheck();
+        if (m_canLowLevel){
+            fullformat->setChecked(true);
+            userFeedBack += i18n( "Program fdformat found." );
+        }
+        else {
+            fullformat->setDisabled(true);
+            quick->setChecked(true);
+            userFeedBack += i18n( "Program fdformat <b>not found</b>. Full formatting <b>disabled</b>." );
+        }
+        
 	verifylabel = new QCheckBox( buttongroup, "RadioButton_4" );
 	verifylabel->setText(i18n( "&Verify integrity" ));
 	verifylabel->setChecked(true);
@@ -134,10 +192,9 @@ FloppyData::FloppyData(QWidget * parent, const char * name)
         QHBoxLayout* h2 = new QHBoxLayout( v2 );
         h2->addSpacing( 20 );
 
-	lineedit = new QLineEdit( buttongroup, "Lineedit" );
-	lineedit->setText(i18n( "KDE Floppy") );
+	lineedit = new KLineEdit( buttongroup, "Lineedit" );
+	lineedit->setText(i18n( "Volume label, maximal 11 characters", "KDE Floppy" ) );
 	lineedit->setMaxLength(11);
-        lineedit->setMinimumWidth( lineedit->sizeHint().width() );
         h2->addWidget( lineedit, AlignRight );
 
 	connect(labellabel,SIGNAL(toggled(bool)),lineedit,SLOT(setEnabled(bool)));
@@ -147,6 +204,8 @@ FloppyData::FloppyData(QWidget * parent, const char * name)
 	formatbutton = new KPushButton( this, "PushButton_3" );
 	formatbutton->setText(i18n( "&Format") );
 	formatbutton->setAutoRepeat( false );
+        if (!numFileSystems)
+            formatbutton->setDisabled(false); // We have not any helper program for creating any file system
 	connect(formatbutton,SIGNAL(clicked()),this,SLOT(format()));
         v3->addWidget( formatbutton );
 
@@ -160,7 +219,7 @@ FloppyData::FloppyData(QWidget * parent, const char * name)
 	helpbutton->setPopup(helpMenu->menu());
 	v3->addWidget( helpbutton );
 
-	quitbutton = new KPushButton( KGuiItem( i18n( "&Quit" ), "exit" ), this );
+	quitbutton = new KPushButton( KStdGuiItem::quit(), this );
 	quitbutton->setAutoRepeat( false );
 	connect(quitbutton,SIGNAL(clicked()),this,SLOT(quit()));
 	 v3->addWidget( quitbutton );
@@ -168,28 +227,32 @@ FloppyData::FloppyData(QWidget * parent, const char * name)
         ml->addSpacing( 10 );
 
 	frame = new QLabel( this, "NewsWindow" );
-	frame->setMinimumHeight( 50 );
 	frame->setFrameStyle(QFrame::Panel | QFrame::Sunken);
-	frame->setAlignment(AlignCenter|WordBreak|ExpandTabs);
+	frame->setAlignment(WordBreak|ExpandTabs);
+
+        QString frameText( userFeedBack );
+        frameText.prepend( "<qt>" );
+        frameText.append( "</qt>" );
+        frame->setText( frameText );
+        
         ml->addWidget( frame );
 
 	progress = new KProgress( this, "Progress" );
-        progress->setMinimumHeight( 30 );
+        progress->setDisabled( true );
         ml->addWidget( progress );
 
 	readSettings();
 	setWidgets();
 
-	findExecutables();
-
-    int maxW = QMAX( deviceComboBox->sizeHint().width(),
-                     densityComboBox->sizeHint().width() );
-    maxW = QMAX( maxW, filesystemComboBox->sizeHint().width() );
-    deviceComboBox->setMinimumWidth( maxW );
-    densityComboBox->setMinimumWidth( maxW );
-    filesystemComboBox->setMinimumWidth( maxW );
-
-    setMinimumSize(minimumSizeHint());
+    if (!numFileSystems) {
+        QString errorMessage;
+        errorMessage += "<qt>";
+        errorMessage += i18n("KFloppy cannot find any of the needed programs for creating file systems; please check your installation.<br><br>Log:");
+        errorMessage += "<br>";
+        errorMessage += userFeedBack;
+        errorMessage += "</qt>";
+        KMessageBox::error( this, errorMessage );
+    }
 }
 
 
@@ -231,9 +294,9 @@ bool FloppyData::findDevice()
   {
     drive=1;
   }
-  
+
   blocks=-1;
-    
+
     if( densityComboBox->currentText() == i18n("3.5\" 1.44MB")){
       blocks = 1440;
     }
@@ -249,8 +312,12 @@ bool FloppyData::findDevice()
     if( densityComboBox->currentText() == i18n("5.25\" 360KB")){
       blocks = 360;
       }
-  
-  
+#if defined(ANY_LINUX)
+    else { // For Linux, anything else is Auto
+        blocks = 0;
+    }
+#endif
+
   return true;
 }
 
@@ -264,34 +331,10 @@ bool FloppyData::setInitialDevice(const QString& dev)
 
   bool ok = (drive>=0);
   if (ok)
-    deviceComboBox->setCurrentItem(drive);  
+    deviceComboBox->setCurrentItem(drive);
   return ok;
 }
 
-
-void FloppyData::findExecutables()
-{
-	bool fruitful = true ;
-	fruitful &= FDFormat::runtimeCheck();
-	fruitful &= FATFilesystem::runtimeCheck();
-#ifdef ANY_BSD
-	fruitful &= UFSFilesystem::runtimeCheck();
-#else
-#ifdef ANY_LINUX
-	fruitful &= Ext2Filesystem::runtimeCheck();
-#else
-	fruitful = false;
-#endif
-#endif
-	
-  if (!fruitful) {
-    formatbutton->setEnabled(false);
-    KMessageBox::error(this, 
-    	i18n("KFloppy cannot find the support programs needed "
-		"for sensible operation."));
-  }
-}
-
 void FloppyData::quit(){
   if (formatActions) formatActions->quit();
   writeSettings();
@@ -313,13 +356,14 @@ void FloppyData::setEnabled(bool b)
   filesystemComboBox->setEnabled(b);
   buttongroup->setEnabled(b);
   quick->setEnabled(b);
-  fullformat->setEnabled(b);
+  fullformat->setEnabled(b && m_canLowLevel);
   verifylabel->setEnabled(b);
   labellabel->setEnabled(b);
   lineedit->setEnabled(b);
   helpbutton->setEnabled(b);
   quitbutton->setEnabled(b);
   formatbutton->setEnabled(b);
+  progress->setDisabled( b ); // The other way round!
 }
 
 void FloppyData::reset()
@@ -328,7 +372,7 @@ void FloppyData::reset()
 
   formating = false;
   quickerase = false;
-  
+
   if (formatActions)
   {
     formatActions->quit();
@@ -354,10 +398,10 @@ void FloppyData::format(){
   }
 
   frame->clear();
-  
-  if (KMessageBox::warningContinueCancel(0, 
+
+  if (KMessageBox::warningContinueCancel(0,
    i18n("Formatting will erase all data on the disk.\n"
-        "Are you sure you wish to proceed?"), i18n("Proceed?") ) != 
+        "Are you sure you wish to proceed?"), i18n("Proceed?") ) !=
 	KMessageBox::Continue)
         {
 	return;
@@ -374,7 +418,7 @@ void FloppyData::format(){
 
 	if (formatActions) delete formatActions;
 	formatActions = new KFActionQueue(this);
-	
+
 	connect(formatActions,SIGNAL(status(const QString &,int)),
 		this,SLOT(formatStatus(const QString &,int)));
 	connect(formatActions,SIGNAL(done(KFAction *,bool)),
@@ -393,8 +437,8 @@ void FloppyData::format(){
 		f->configure(verifylabel->isChecked());
 		formatActions->queue(f);
 	}
-	
-	if (filesystemComboBox->currentItem() == 0)
+
+	if ( filesystemComboBox->currentText() == i18n("DOS") )
 	{
 		FATFilesystem *f = new FATFilesystem(this);
 		connect(f,SIGNAL(status(const QString &,int)),
@@ -407,21 +451,13 @@ void FloppyData::format(){
 		f->configureDevice(drive,blocks);
 		formatActions->queue(f);
 	}
-	else if (filesystemComboBox->currentItem() == 1)
-	{
-		FloppyAction *f = 0L;
 #ifdef ANY_LINUX
-		Ext2Filesystem *e2f = new Ext2Filesystem(this);
-		e2f->configure(verifylabel->isChecked(),
+	else if ( filesystemComboBox->currentText() == i18n("ext2") )
+	{
+		Ext2Filesystem *f = new Ext2Filesystem(this);
+		f->configure(verifylabel->isChecked(),
 			labellabel->isChecked(),
 			lineedit->text());
-		f=e2f;
-#else
-#ifdef ANY_BSD
-		f = new UFSFilesystem(this);
-#endif
-#endif
-		
 		if (f)
 		{
 			connect(f,SIGNAL(status(const QString &,int)),
@@ -432,17 +468,55 @@ void FloppyData::format(){
 			formatActions->queue(f);
 		}
 	}
+#endif
+
+#ifdef ANY_BSD
+	else if ( filesystemComboBox->currentText() == i18n("UFS") )
+	{
+		FloppyAction *f = new UFSFilesystem(this);
+
+		if (f)
+		{
+			connect(f,SIGNAL(status(const QString &,int)),
+				this,SLOT(formatStatus(const QString &,int)));
+			connect(f,SIGNAL(done(KFAction *,bool)),
+				this,SLOT(reset()));
+			f->configureDevice(drive,blocks);
+			formatActions->queue(f);
+		}
+	}
+#endif
+
+#ifdef ANY_LINUX
+	else if ( filesystemComboBox->currentText() == i18n("Minix") )
+	{
+		MinixFilesystem *f = new MinixFilesystem(this);
+		f->configure(verifylabel->isChecked(),
+			labellabel->isChecked(),
+			lineedit->text());
+		if (f)
+		{
+			connect(f,SIGNAL(status(const QString &,int)),
+				this,SLOT(formatStatus(const QString &,int)));
+			connect(f,SIGNAL(done(KFAction *,bool)),
+				this,SLOT(reset()));
+			f->configureDevice(drive,blocks);
+			formatActions->queue(f);
+		}
+	}
+#endif
+
+
 
-	
-		
 	formatActions->exec();
 }
 
 void FloppyData::formatStatus(const QString &s,int p)
 {
+    kdDebug(2002) << "FloppyData::formatStatus: " << s << " : "  << p << endl;
 	if (!s.isEmpty())
 		frame->setText(s);
-	
+
 	if ((0<=p) && (p<=100))
 		progress->setValue(p);
 }
@@ -472,7 +546,7 @@ void FloppyData::readStdout(KProcess *, char *buffer, int buflen)
   if (increment)
   {
     counter ++;
-    progress->setValue(counter);     
+    progress->setValue(counter);
   }
 
   kdDebug(2002) << "STDOUT: " << formatstring << endl;
@@ -507,19 +581,19 @@ void FloppyData::errslot(){
 
   if(errstring.contains("ioctl(FDFMTBEG)")){
 
-    QString str = i18n(
+    QString str = i 18n(
 		"Cannot access floppy or floppy drive.\n"
 		"Please insert a floppy and make sure that you "
 		"have selected a valid floppy drive.");
 
     KMessageBox::error(this, str);
-    
+
     reset();
     return;
 
   }
 
-  QString str = i18n("Cannot format: %1\n%2").arg(drive).arg(errstring);
+  QString str = i 18n("Cannot format: %1\n%2").arg(drive).arg(errstring);
 
   KMessageBox::error(this, str);
 
@@ -541,10 +615,10 @@ void FloppyData::readfsStdout(KProcess *, char *buffer, int buflen){
   fsstring += mybuffer;
   if( fsstring.find('\n') == -1)
     return;
-  
+
   QString string ;
   QString newstring = fsstring;
-  
+
 
   int i;
 
@@ -558,7 +632,7 @@ void FloppyData::readfsStdout(KProcess *, char *buffer, int buflen){
     if(findKeyWord(mystring,"BBF ")){
       int bblock = mystring.left(8).toInt();
       QString mstr;
-      mstr = i18n("Block %1 is bad. Continuing...").arg(bblock);
+      mstr = i 18n("Block %1 is bad. Continuing...").arg(bblock);
       frame->setText(mstr);
     }
 
@@ -580,12 +654,12 @@ void FloppyData::readfsStdout(KProcess *, char *buffer, int buflen){
     if(findKeyWord(fsstring,"EXIT"))
       counter ++;
 
-    
+
   }
 
   fsstring = newstring;
 
-  progress->setValue(counter);     
+  progress->setValue(counter);
 
   kdDebug(2002) << "STDOUT: " << mybuffer << endl;
 }
@@ -636,20 +710,20 @@ void FloppyData::fserrslot(){
 
   if(fserrstring.contains("No such device")){
 
-    QString str = i18n(
+    QString str = i 18n(
 		"Cannot access floppy or floppy drive.\n"
 		"Please insert a floppy and make sure that you "
  		"have selected a valid floppy drive.");
 
     KMessageBox::sorry(this, str);
-    
+
     reset();
     return;
   }
 
   reset();
 
-  QString str = i18n("Cannot create a filesystem on: %1\n%2")
+  QString str = i 18n("Cannot create a filesystem on: %1\n%2")
 	      .arg(drive).arg(fserrstring);
 
   KMessageBox::error(this, str);
@@ -669,14 +743,14 @@ void FloppyData::cf2done(){
   lcquick = quickerase;
 
   mytimer->stop();
-  
+
   reset();
-  
+
   if(abort)
     return;
 
   if(!lcquick){
-    QString str = i18n(
+    QString str = i 18n(
 		"The floppy was successfully formatted.\n"
 		"Blocks marked bad: %1\n"
 		"Raw Capacity: %2\n")
@@ -686,7 +760,7 @@ void FloppyData::cf2done(){
     KMessageBox::information(this, str);
   }
   else{
-    QString str = i18n("All files were successfully erased.");
+    QString str = i 18n("All files were successfully erased.");
 
     KMessageBox::information(this, str);
   }
@@ -700,7 +774,7 @@ int FloppyData::findKeyWord(QString & string,const QString & word){
 
   QString wordstring = word;
   len = wordstring.length();
-    
+
   while( (index = string.find(word)) >= 0)
   {
     count++;
@@ -764,9 +838,9 @@ void FloppyData::setWidgets(){
 
   labellabel->setChecked(labelconfig);
   verifylabel->setChecked(verifyconfig);
-  quick->setChecked(quickformatconfig);
+  quick->setChecked(quickformatconfig || !m_canLowLevel);
 
-  fullformat->setChecked(!quickformatconfig);
+  fullformat->setChecked(!quickformatconfig && m_canLowLevel);
   lineedit->setText(labelnameconfig);
 
   for(int i = 0 ; i < deviceComboBox->count(); i++){
@@ -788,4 +862,4 @@ void FloppyData::setWidgets(){
   }
 }
 
-
+#include "floppy.moc"
