@@ -1,7 +1,7 @@
 /*
  * portable.cpp
  *
- * $Id: portable.cpp,v 1.84.2.1 2004/03/29 23:46:09 mueller Exp $
+ * $Id: portable.cpp,v 1.93 2005/02/14 16:53:47 waba Exp $
  *
  * Copyright (c) 1999 Paul Campbell <paul@taniwha.com>
  *
@@ -95,6 +95,7 @@
 #include <kconfig.h>
 #include <kstandarddirs.h>
 #include <kprocess.h>
+#include <krichtextlabel.h>
 
 // ibm specific stuff
 extern "C"{
@@ -510,11 +511,13 @@ acpi_read(apm_info *ap)
 		for (int i = 0; i < nrates; i++)
 			rate += saved_rate[i];
 		rate = (rate+2*saved_rate[0])/(nrates+2);		// weight it slighly
-		ap->battery_time = ((rate==0||part==0)?0 : 60*part/rate);
+		ap->battery_time = (rate==0 ? -1 : 60*part/rate);
 	}
 	ap->battery_percentage = (total==0?0:100*part/total);
-	if (!present)
+	if (!present) {
 		ap->battery_percentage = -1;
+		ap->battery_time = -1;
+	}
 	ap->apm_flags = 0;
 	return(ret);
 }
@@ -574,7 +577,8 @@ has_software_suspend(int type)
 	if (known != last_seed) {
 		known = last_seed;
 		available = 0;
-		present = (::access("/proc/sys/kernel/swsusp", F_OK) == 0 &&
+		present = (((::access("/proc/sys/kernel/swsusp", F_OK) == 0) ||
+                            (::access("/proc/software_suspend", F_OK) == 0)) &&
 		           ::access("/usr/sbin/hibernate", F_OK) == 0);
 		if (present) {
 			if (::getuid() == 0) {	// running as root
@@ -919,13 +923,11 @@ KActiveLabel *laptop_portable::no_power_management_explanation(QWidget *parent)
 {
 	if (access("/proc/acpi", F_OK) == 0) {	// probably has default kernel ACPI installed
 		KActiveLabel* explain = new KActiveLabel(i18n("Your computer seems to have a partial ACPI installation. ACPI was probably enabled, but some of the sub-options were not - you need to enable at least 'AC Adaptor' and 'Control Method Battery' and then rebuild your kernel."), parent);
-      		explain->setMinimumSize(explain->sizeHint());       
 		return(explain);
 	}
 
 	KActiveLabel* explain = new KActiveLabel(i18n("Your computer doesn't have the Linux APM (Advanced Power Management) or ACPI software installed, or doesn't have the APM kernel drivers installed - check out the <a href=\"http://www.linuxdoc.org/HOWTO/Laptop-HOWTO.html\">Linux Laptop-HOWTO</a> document for information on how to install APM."), parent);
  
-     	explain->setMinimumSize(explain->sizeHint());       
 	return(explain);
 }
 
@@ -935,17 +937,20 @@ KActiveLabel *laptop_portable::no_power_management_explanation(QWidget *parent)
 QLabel *laptop_portable::how_to_do_suspend_resume(QWidget *parent)
 {
 	if (::has_apm()) {
- 		QLabel* note = new QLabel(i18n("\nIf you make /usr/bin/apm setuid then you will also\nbe able to choose 'suspend' and 'standby' in the\nabove dialog - check out the help button below to\nfind out how to do this"), parent);
-        	note->setMinimumSize(note->sizeHint()); 
+                // TODO: remove linefeed from string, can't do it right now coz we have a string freeze
+ 		QLabel* note = new KRichTextLabel(i18n("\nIf you make /usr/bin/apm setuid then you will also "
+						"be able to choose 'suspend' and 'standby' in the above "
+						"dialog - check out the help button below to find out "
+						"how to do this").replace("\n", QString::null), parent);
 		return(note);
 	}
 	if (::has_acpi()) {
- 		QLabel* note = new QLabel(i18n("\nYou may need to enable ACPI suspend/resume in the ACPI panel"), parent);
-        	note->setMinimumSize(note->sizeHint()); 
+                // TODO: remove linefeed from string, can't do it right now coz we have a string freeze
+ 		QLabel* note = new KRichTextLabel(i18n("\nYou may need to enable ACPI suspend/resume in the ACPI panel").replace("\n", QString::null), parent);
 		return(note);
 	}
- 	QLabel* note = new QLabel(i18n("\nYour system does not support suspend/standby"), parent);
-        note->setMinimumSize(note->sizeHint()); 
+        // TODO: remove linefeed from string, can't do it right now coz we have a string freeze
+ 	QLabel* note = new KRichTextLabel(i18n("\nYour system does not support suspend/standby").replace("\n", QString::null), parent);
 	return(note);
 }
 
@@ -1142,14 +1147,14 @@ laptop_portable::extra_config(QWidget *wp, KConfig *, QVBoxLayout *top_layout)
 	if (laptop_portable::has_apm(1) || laptop_portable::has_acpi(1))
 		return;
 	if (laptop_portable::has_apm(0)) {
-      		QLabel* explain = new QLabel( i18n("Your system has APM installed but may not be able to use all\n"
-					           "of its features without further setup - look in the 'APM Config'\n"
+      		QLabel* explain = new KRichTextLabel( i18n("Your system has APM installed but may not be able to use all "
+					           "of its features without further setup - look in the 'APM Config' "
 						   "tab for information about setting up APM for suspend and resume"), wp);
-      		top_layout->addWidget(explain, 0);
+		top_layout->addWidget(explain, 0);
 	}
 	if (laptop_portable::has_acpi(0)) {
-      		QLabel* explain = new QLabel( i18n("Your system has ACPI installed but may not be able to use all\n"
-					           "of its features without further setup - look in the 'ACPI Config'\n"
+      		QLabel* explain = new KRichTextLabel( i18n("Your system has ACPI installed but may not be able to use all "
+					           "of its features without further setup - look in the 'ACPI Config' "
 						   "tab for information about setting up ACPI for suspend and resume"), wp);
       		top_layout->addWidget(explain, 0);
 	}
@@ -1458,7 +1463,7 @@ laptop_portable::set_brightness(bool blank, int val)	// val = 0-255 255 brightes
 			val = 0;
 		if (val > 255)
 			val = 255;
-		val = (val+7)>>5;
+		val >>= 5;
 		if (val == 0 && !blank)
 			val = 1;
 		snprintf(tmp, sizeof(tmp), "%d", val&7);
@@ -1606,7 +1611,7 @@ static int get_cpufreq_sysfs_state(QStringList &states, int &current, const QStr
 	int count = 0;
 	if(!f.atEnd()) {
 		f.readLine(buffer, 1024);
-		QStringList l = QStringList::split(' ', buffer, false);
+		QStringList l = QStringList::split(' ', buffer.stripWhiteSpace(), false);
 		for(unsigned int i = 0; i < l.size(); ++i, ++count) {
 			states.append(l[i].stripWhiteSpace());
 			if(states[i] == cur)
@@ -2004,7 +2009,8 @@ laptop_portable::get_button(LaptopButton l)	// true if a button is pressed
 //	returns 1 if we support power management
 //
 
-#include <iostream.h>
+#include <iostream>
+using namespace std;
 
 //
 //	something changed maybe we need to check out environment again
@@ -2102,8 +2108,6 @@ int laptop_portable::has_standby()
 //
 int laptop_portable::has_hibernation()
 {
-	struct stat s;
-
 	if (::access(PATH_TPCTL, X_OK)==0)
 		return(1);
 	return(0);
@@ -2140,7 +2144,6 @@ KActiveLabel *laptop_portable::no_power_management_explanation(QWidget *parent)
     explain = new KActiveLabel(i18n("APM has most likely been disabled."), parent);
   }
   
-  explain->setMinimumSize(explain->sizeHint());       
   return(explain);
 }
 
@@ -2150,7 +2153,6 @@ KActiveLabel *laptop_portable::no_power_management_explanation(QWidget *parent)
 QLabel *laptop_portable::how_to_do_suspend_resume(QWidget *parent)
 {
  	QLabel* note = new QLabel(i18n(" "), parent);
-        note->setMinimumSize(note->sizeHint()); 
 	return(note);
 }
 
@@ -2243,7 +2245,7 @@ laptop_portable::apm_set_mask(bool , bool )
 //	adds extra widgets to the battery panel
 //
 void
-laptop_portable::extra_config(QWidget *parent, KConfig *config, QVBoxLayout *layout)
+laptop_portable::extra_config(QWidget * /*parent*/, KConfig * /*config*/, QVBoxLayout * /*layout*/)
 {
 	// INSERT HERE
 }
@@ -2312,7 +2314,7 @@ laptop_portable::has_brightness()
 }
 
 void 
-laptop_portable::set_brightness(bool blank, int val)	// val = 0-255 255 brightest, 0 means dimmest (if !blank it must be still visible), 
+laptop_portable::set_brightness(bool /*blank*/, int /*val*/)	// val = 0-255 255 brightest, 0 means dimmest (if !blank it must be still visible), 
 {
 	// INSERT HERE
 }
@@ -2393,13 +2395,13 @@ laptop_portable::get_battery_status(int &num_batteries, QStringList &names, QStr
 }
 
 bool
-laptop_portable::has_software_suspend(int type)
+laptop_portable::has_software_suspend(int /*type*/)
 {
 	return false; // (::has_software_suspend(type));
 }
 
 void
-laptop_portable::software_suspend_set_mask(bool hibernate)
+laptop_portable::software_suspend_set_mask(bool /*hibernate*/)
 {
 	// software_suspend_is_preferred = hibernate;
 }
@@ -2547,7 +2549,6 @@ KActiveLabel *laptop_portable::no_power_management_explanation(QWidget *parent)
     explain = new KActiveLabel(i18n("APM has most likely been disabled."), parent);
   }
   
-  explain->setMinimumSize(explain->sizeHint());       
   return(explain);
 }
 
@@ -2558,7 +2559,6 @@ QLabel *laptop_portable::how_to_do_suspend_resume(QWidget *parent)
 {
 	// INSERT HERE
        QLabel* note = new QLabel(i18n(" "), parent);
-        note->setMinimumSize(note->sizeHint()); 
        return(note);
 }
 
@@ -2843,7 +2843,6 @@ KActiveLabel *laptop_portable::no_power_management_explanation(QWidget *parent)
 {
 	KActiveLabel* explain = new KActiveLabel(i18n("Your computer or operating system is not supported by the current version of the\nKDE laptop control panels. If you want help porting these panels to work with it\nplease contact paul@taniwha.com."), parent);
 	// INSERT HERE
-      	explain->setMinimumSize(explain->sizeHint());       
 	return(explain);
 }
 
@@ -2854,7 +2853,6 @@ QLabel *laptop_portable::how_to_do_suspend_resume(QWidget *parent)
 {
  	QLabel* note = new QLabel(i18n(" "), parent);
 	// INSERT HERE
-        note->setMinimumSize(note->sizeHint()); 
 	return(note);
 }
 
