@@ -301,14 +301,17 @@ PowerConfig::PowerConfig (QWidget * parent, const char *name)
 
       hlay->addStretch(1);
 
-      QLabel* explain = new QLabel(i18n("This panel configures the behavior of the automatic power-down feature - it works as a sort of extreme\n"
-			                "screen saver. You can configure different timeouts and types of behavior depending on whether or not\n"
-					"your laptop is plugged in to the mains supply."), this);
-      top_layout->addWidget(explain, 0, Qt::AlignLeft);
+      QLabel* explain = new QLabel(i18n("This panel configures the behavior of the automatic power-down feature - "
+					"it works as a sort of extreme screen saver. You can configure different "
+					"timeouts and types of behavior depending on whether or not your laptop is "
+					"plugged in to the mains supply."), this );
+      explain->setAlignment( Qt::WordBreak );
+      top_layout->addWidget(explain);
 
       if (can_standby) {
-	QLabel* explain3 = new QLabel(i18n("Different laptops may respond to 'standby' in different ways - in many it's only a temporary\n"
-				           "state and may not be useful for you."), this);
+	QLabel* explain3 = new QLabel(i18n("Different laptops may respond to 'standby' in different ways - in many "
+					"it is only a temporary state and may not be useful for you."), this);
+	explain3->setAlignment( Qt::WordBreak );
 	top_layout->addWidget(explain3, 0, Qt::AlignLeft);
       }
 
@@ -557,6 +560,7 @@ void PowerConfig::defaults()
 		powerValThrottle->setEnabled(power_throttle_enabled);
 	}
   }
+  changed(true);
 }
 
 int  PowerConfig::getNoPower()
